/*
  UTFT.h - Multi-Platform library support for Color TFT LCD Boards
  Copyright (C)2015 Rinky-Dink Electronics, Henning Karlsen. All right reserved
  
  This library is the continuation of my ITDB02_Graph, ITDB02_Graph16
  and RGB_GLCD libraries for Arduino and chipKit. As the number of 
  supported display modules and controllers started to increase I felt 
  it was time to make a single, universal library as it will be much 
  easier to maintain in the future.

  Basic functionality of this library was origianlly based on the 
  demo-code provided by ITead studio (for the ITDB02 modules) and 
  NKC Electronics (for the RGB GLCD module/shield).

  This library supports a number of 8bit, 16bit and serial graphic 
  displays, and will work with both Arduino, chipKit boards and select 
  TI LaunchPads. For a full list of tested display modules and controllers,
  see the document UTFT_Supported_display_modules_&_controllers.pdf.

  When using 8bit and 16bit display modules there are some 
  requirements you must adhere to. These requirements can be found 
  in the document UTFT_Requirements.pdf.
  There are no special requirements when using serial displays.

  You can find the latest version of the library at 
  http://www.RinkyDinkElectronics.com/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the CC BY-NC-SA 3.0 license.
  Please see the included documents for further information.

  Commercial use of this library requires you to buy a license that
  will allow commercial use. This includes using the library,
  modified or not, as a tool to sell products.

  The license applies to all part of the library including the 
  examples and tools supplied with the library.
*/

#ifndef UTFT_h
#define UTFT_h

#include "system_definitions.h"

#define UTFT_VERSION	282

#define LEFT 0
#define RIGHT 9999
#define CENTER 9998

#define PORTRAIT 0
#define LANDSCAPE 1

#define swap(type, i, j) {type t = i; i = j; j = t;}
#define fontbyte(x) cfont.font[x]  
#define bitmapdatatype unsigned short*

//#define RS_COM SYS_PORTS_PinClear(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15);
//#define RS_PARA SYS_PORTS_PinSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15);
//#define WR_ON SYS_PORTS_PinSet(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_4);
//#define WR_OFF SYS_PORTS_PinClear(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_4);
#define CS_DISEN SYS_PORTS_PinSet(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_1);
#define CS_EN SYS_PORTS_PinClear(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_1);
#define RST_ON SYS_PORTS_PinSet(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_2);
#define RST_OFF SYS_PORTS_PinClear(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_2);

#define ILI9327			1

//*********************************
// COLORS
//*********************************
// VGA color palette
#define VGA_BLACK		0x0000
#define VGA_WHITE		0xFFFF
#define VGA_RED			0xF800
#define VGA_GREEN		0x0400
#define VGA_BLUE		0x001F
#define VGA_SILVER		0xC618
#define VGA_GRAY		0x8410
#define VGA_MAROON		0x8000
#define VGA_YELLOW		0xFFE0
#define VGA_OLIVE		0x8400
#define VGA_LIME		0x07E0
#define VGA_AQUA		0x07FF
#define VGA_TEAL		0x0410
#define VGA_NAVY		0x0010
#define VGA_FUCHSIA		0xF81F
#define VGA_PURPLE		0x8010
#define VGA_TRANSPARENT	0xFFFFFFFF


struct _current_font
{
	uint8_t* font;
	uint8_t x_size;
	uint8_t y_size;
	uint8_t offset;
	uint8_t numchars;
}cfont;

		void	InitLCD(uint8_t orientation);
		void	clrScr();
		void	drawPixel(int x, int y);
		void	drawLine(int x1, int y1, int x2, int y2);
		void	fillScr(uint8_t r, uint8_t g, uint8_t b);
		void	fillScr_w(uint16_t color);
		void	drawRect(int x1, int y1, int x2, int y2);
		void	drawRoundRect(int x1, int y1, int x2, int y2);
		void	fillRect(int x1, int y1, int x2, int y2);
		void	fillRoundRect(int x1, int y1, int x2, int y2);
		void	drawCircle(int x, int y, int radius);
		void	fillCircle(int x, int y, int radius);
		void	setColor(uint8_t r, uint8_t g, uint8_t b);
		void	setColor_w(uint16_t color);
		uint16_t	getColor();
		void	setBackColor(uint8_t r, uint8_t g, uint8_t b);
		void	setBackColo_wr(uint32_t color);
		uint16_t	getBackColor();
		void	print(uint8_t *st, int x, int y, int deg);
//		void	print(String st, int x, int y, int deg=0);
		void	printNumI(long num, int x, int y, int length, char filler);
		void	printNumF(double num, uint8_t dec, int x, int y, char divider, int length, char filler);
		void	setFont(uint8_t* font);
        uint8_t* getFont();
		uint8_t	getFontXsize();
		uint8_t	getFontYsize();
		void	drawBitmap(int x, int y, int sx, int sy, bitmapdatatype data, int scale);
		void	drawBitmap_r(int x, int y, int sx, int sy, bitmapdatatype data, int deg, int rox, int roy);
//		void	lcdOff();
//		void	lcdOn();
//		void	setContrast(char c);
//		int		getDisplayXSize();
//		int		getDisplayYSize();
//		void	setBrightness(byte br);
//		void	setDisplayPage(byte page);
//		void	setWritePage(byte page);

/*
	The functions and variables below should not normally be used.
	They have been left publicly available for use in add-on libraries
	that might need access to the lower level functions of UTFT.

	Please note that these functions and variables are not documented
	and I do not provide support on how to use them.
*/
		uint8_t			fch, fcl, bch, bcl;
		uint8_t			orient;
		long			disp_x_size, disp_y_size;
		uint8_t			display_model, display_transfer_mode, display_serial_mode;
		bool			_transparent;

		void LCD_Writ_Bus(uint8_t VH, uint8_t VL, uint8_t mode);
		void LCD_Write_COM(uint8_t VL);
		void LCD_Write_BUS( uint8_t VH, uint8_t VL);
		void LCD_Write_DATA( uint8_t VL);
		void LCD_Write_COM_DATA(char com1,int dat1);
		void _hw_special_init();
		void setPixel(uint16_t color);
        uint16_t getPixel(int x, int y);
		void drawHLine(int x, int y, int l);
		void drawVLine(int x, int y, int l);
		void printChar(uint8_t c, int x, int y);
		void setXY(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
        void getXY(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
		void clrXY();
		void rotateChar(uint8_t c, int x, int y, int pos, int deg);
//		void _set_direction_registers(uint8_t mode);
//		void _fast_fill_16(int ch, int cl, long pix);
		void _fast_fill_8(int ch, long pix);
		void _convert_float(uint8_t *buf, double num, int width, uint8_t prec);
        void Delay10us(unsigned long dwCount);
        void DelayMs(unsigned short int ms);

#endif